/* 	This file is part of EasyStream,
                an open-source cross-platform VDR-Client using libvlc

                Copyright (c) :[2011-2013] Werner Sigrist
                Contact e-mail: Werner Sigrist <sig@sigvdr>
                Program URL   : http://www.sigvdr.de/mediawiki/index.php?title=QtLibVlc

         This package is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2 of the License, or
         (at your option) any later version.

         This package is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.

         You should have received a copy of the GNU General Public License
         along with this package; if not, see <http://www.gnu.org/licenses/>
*/
#include "player.h"
#include <QBitmap>
#include <QToolButton>
#include <QTextCodec>
#include <QPixmap>
#include <QRegion>
#ifdef Q_OS_MAC
#include <AppKit/NSView.h>
#endif

#include "../func.h"

// libvlc >= 1.1.0

Player::Player(QWidget *parent,bool /*videoMode*/) :
    QX11Widget(parent)
{
//    buttonRegion = new QRegion();
    slVolume = 0;
    slPosition = 0;
    slFilesIndex = 0;
    playerRuns = false;
    playerRunsExtern = false;
    playerPauseState = false;
    //isFullScreen = false;
    posTimer = new QTimer;
    maxValue = MAXVALUEDEFAULT;

    videoPosFileName = getBaseFileName()+".pos";
}
Player::~Player()
{
    if ( playerRuns) {
        stopAndSavePosition();
    }
    if ( playerRunsExtern) {
        stopExtern();
    }
}

void Player::setConfig(Optionen *config){
    conf = config;
}

/*
    Player Interface +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::play(QStringList *fileNames){
    playerRunLive = false;
    if (playerRuns) {
        stopAndSavePosition();
        //playNext();
        //return;
    }
    currentFiles.clear();
    for ( int i = 0; i < fileNames->count(); i++)
        currentFiles << fileNames->at(i);
    if (playerRunsExtern) {
        stopExtern();
    }
    if (conf->cBPlayer->isChecked() ) {
        playExtern();
    } else {
        playIntern();
    }

}

void Player::playLiveStream(QString fileName){
    playerRunLive = true;
    if (playerRuns) {
        stopAndSavePosition();
        //playNext();
        //return;
    }
    if (playerRunsExtern) {
        stopExtern();
    }
//    setFileName(fileName);
    currentFiles.clear();
    currentFiles << fileName;
    conf->logger("player.cpp->play() "+QString::number(currentFiles.count())+" Datei(en)");
    if (conf->cBPlayer->isChecked() ) {
        playExtern();
    } else {
        playIntern();
    }
}
QString Player::fileName(void){
    if (currentFiles.count() == 0) return "";
    return currentFiles[0];
}

bool Player::isRunning(void){
    return playerRuns;
}
bool Player::isRunningLive(void){
    return playerRunLive & playerRuns;
}
bool Player::isRunningVideo(void){
    return (!playerRunLive) & playerRuns;
}

/*
    external Player +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::playExtern(void){
    QStringList args;
    QString options = conf->mPlayerOptions->toPlainText().trimmed();
    if ( options.size() ){
        args << options.split("\n"); //.replace("\n"," ");
    }
    options = conf->mPlayerAddOptions->toPlainText().trimmed();
    if ( options.size() ){
        args << options.split("\n"); //.replace("\n"," ");
    }

    args << currentFiles;
    QString cmdPlay = conf->ePlayer->text();
    externPlayer = new QProcess();
    externPlayer->start(cmdPlay,args,QIODevice::WriteOnly | QIODevice::Unbuffered );
    playerRunsExtern = true;
}

void Player::stopExtern(void){
    if (externPlayer != NULL) {
        externPlayer->close();
        externPlayer = NULL;
        playerRunsExtern = false;
    }
}

/*
    libvlc Player +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::playIntern(void){
    QTextCodec *codec = QTextCodec::codecForName(conf->cBCodecVdr->currentText().toLocal8Bit()); // wg. den Aufnahmen nur so
    QStringList addOptions = conf->mPlayerAddOptions->toPlainText().split("\n");
    sizes.clear();
    mediaLength = 0;
    conf->logger("player.cpp->play()");
    if ( currentFiles.count() == 0) {
        return;
    }

    if (playerRuns) {
        pause();
        return;
    }
    /* Load the VLC engine */
    vlcInstance = libvlc_new (0, NULL);
    /* create a new media list */
    mediaList = libvlc_media_list_new(vlcInstance);
    /* load list of media */
    for (int i = 0; i < currentFiles.count(); i++ ) {
        /* Create a new item */
        QString name = currentFiles[i];
        emit localFileMode(!playerRunLive);
#ifdef Q_OS_MAC //Ist wohl eher ein vlc 2.1 request als MAC
        if ( playerRunLive) {
        media = libvlc_media_new_location (vlcInstance,codec->fromUnicode(name)); 
        } else {
        media = libvlc_media_new_path (vlcInstance,codec->fromUnicode(name));
        }
#else
        media = libvlc_media_new_path (vlcInstance,codec->fromUnicode(name));
#endif
        if ( (!playerRunLive)  & conf->cBCheckRecSize->isChecked() ) {
            libvlc_media_parse(media);
            qint64 size =  libvlc_media_get_duration(media);
            sizes << size;
            mediaLength += size;
        }
        foreach (QString option,addOptions) {
            if ( option.left(2) == "--") option = ":"+option.mid(2);
            libvlc_media_add_option(media, codec->fromUnicode(option));
        }
        /* add to media list */
        libvlc_media_list_add_media(mediaList, media);
        /* No need to keep the media now */
        libvlc_media_release (media);
    }
    mediaListPlayer = libvlc_media_list_player_new(vlcInstance);
    /* Create a media player playing environement */
    mediaPlayer = libvlc_media_player_new(vlcInstance);

    libvlc_media_list_player_set_media_list(mediaListPlayer, mediaList);
    libvlc_media_list_player_set_media_player(mediaListPlayer, mediaPlayer);
    setPositionInt(0);
    mediaListPos = readSavedPosition();
    mediaListCount = currentFiles.count();
    slFilesIndex->setMaximum(mediaListCount-1);
    slFilesIndex->setValue(mediaListPos.index);
    emit currentFileName(currentFiles[mediaListPos.index]);

    // in Fenster einpassen
#ifdef Q_OS_UNIX
#ifdef Q_WS_X11
    libvlc_media_player_set_xwindow (mediaPlayer, this->winId());
    libvlc_video_set_key_input(mediaPlayer,0);
    libvlc_video_set_mouse_input(mediaPlayer,0);
#endif
#endif
#ifdef Q_OS_WIN
    /* or on windows */
    libvlc_media_player_set_hwnd (mediaPlayer, this->winId());
    libvlc_video_set_key_input(mediaPlayer,0);
    libvlc_video_set_mouse_input(mediaPlayer,0);
#endif
#ifdef Q_OS_MAC
    /* or on MAC */
    libvlc_media_player_set_nsobject (mediaPlayer, (NSView *) this->winId());
    libvlc_video_set_key_input(mediaPlayer,0);
    libvlc_video_set_mouse_input(mediaPlayer,0);
#endif
    
    
    
    
#if 0
    /* This is a non working code that show how to hooks into a window,
          * if we have a window around */
    libvlc_media_player_set_xdrawable (mediaPlayer, xdrawable);
    /* or on windows */
    libvlc_media_player_set_hwnd (mediaPlayer, hwnd);
    /* or on mac os */
    libvlc_media_player_set_nsobject (mediaPlayer, view);
#endif

    QString deinterlaceMode = conf->getDeinterlaceMode();
    if ( !deinterlaceMode.isEmpty() ) {
        libvlc_video_set_deinterlace(mediaPlayer,deinterlaceMode.toAscii());
    }
    /* play the media_player */
    libvlc_media_list_player_play_item_at_index(mediaListPlayer,mediaListPos.index);


    lTime->setText( timeToString(mediaLength));

    connect(posTimer,SIGNAL(timeout()),this,SLOT(getPosition()));
    posTimer->start(100);
    playerRuns = true;
    emit started();
    setVolume(slVolume->value());
    if ( mediaListPos.setThisFirst ) {
        setPositionFloat(mediaListPos.pos);
        mediaListPos.setThisFirst = false;
    }
}
void Player::stopIntern(void){
    /* Stop playing */
    conf->logger("player.cpp->stop()");
    posTimer->stop();
    if (playerRuns) {
        playerRuns = false;
        conf->logger("player.cpp->stop()-stop");
        if (  libvlc_media_player_is_playing(mediaPlayer) ) {
            libvlc_media_player_stop (mediaPlayer);
        }
        while (  libvlc_media_player_is_playing(mediaPlayer) ) {
        }

        /* Free the media_list_player */
        conf->logger("player.cpp->stop()-mediaListPlayer");
        libvlc_media_list_player_release (mediaListPlayer);
        /* Free the media_player */
        conf->logger("player.cpp->stop()-mediaplayer");
        libvlc_media_player_release (mediaPlayer);

        conf->logger("player.cpp->stop()-Instance");
        libvlc_release (vlcInstance);
        playerPauseState = false;
        conf->logger("player.cpp->stop()-End");

    }
}
void Player::setVolume(int value){
    conf->logger("player.cpp->setVolume("+QString::number(value)+")");
    conf->volume = value;
    if (playerRuns) {
        libvlc_audio_set_volume (mediaPlayer,value);
    }

}

void Player::pause(void){
    conf->logger("player.cpp->pause()");
    if (playerRuns) {
        playerPauseState = !playerPauseState;
        libvlc_media_player_set_pause (mediaPlayer,int(playerPauseState));
        emit paused(playerPauseState);
    }
}

/*
    GET + SET Position and save/read to/from file +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::stopAndSavePosition(void){
    if ( isRunningVideo() ) {
        QString name = currentFiles[0];
        //mediaListPos.pos = libvlc_media_player_get_position (mediaPlayer)*1000000;
        int l = name.size();
        int index = -1;
        QStringList recList = loadFromFile(videoPosFileName).split("\n");
        for (int i = 0; i < recList.count(); i++) {
            if ( recList[i].left(l) == name ) {
                index = i;
                break;
            }
        }
        name += "="+QString::number(mediaListPos.index)+";"+QString::number(mediaListPos.pos);
        if ( index < 0 ) {
            recList << name;
        } else {
            recList[index] = name;
        }
        saveToFile(videoPosFileName,recList.join("\n"));
        int pos = name.lastIndexOf("/");
        if ( pos <  0 ) pos = name.lastIndexOf("\\");
        emit stopped(name.left(pos));
    }
    stopIntern();
}

MediaListPos Player::readSavedPosition(void){
    MediaListPos newPos;
    newPos.pos = 0;
    newPos.index = 0;
    newPos.setThisFirst = false;
    QString name = currentFiles[0];
    if ( !playerRunLive ) {
        int l = name.size();
        int index = -1;
        QStringList recList = loadFromFile(videoPosFileName).split("\n");
        for (int i = 0; i < recList.count(); i++) {
            if ( recList[i].left(l) == name ) {
                index = i;
                break;
            }
        }
        if ( index > -1 ) {
            name = recList[index];
            int pos = name.lastIndexOf("=");
            int endPos = name.lastIndexOf(";");
            if (( pos > -1) & ( endPos > -1)) {
                pos++;
                newPos.index = name.mid(pos,endPos-pos).toInt();
                newPos.pos = name.mid(endPos+1).toFloat();
                newPos.setThisFirst = true;
            }
        }
    }
    return newPos;
}

/*
    GET + SET Position of stream +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::getPosition(void){
    if (playerRuns & !playerRunLive) {
        if ( mediaLength == 0 ) {
            mediaLength = libvlc_media_player_get_length(mediaPlayer) ;
            lTime->setText( timeToString(mediaLength));
        }
        mediaListPos.pos = libvlc_media_player_get_position (mediaPlayer);
        setMediaSliderPosition(mediaListPos.pos);
//        lTimeLeft->setText( timeToString(mediaLength - libvlc_media_player_get_time (mediaPlayer)));
        videoPos();
        lTimeLeft->setMinimumWidth(lTimeLeft->width());
        // libvlc crashed am Ende deshalb einige ms vorher beenden
        if ( (mediaListPos.pos > 0.9999) ) {
            mediaListPos.pos = 0;
            if ( ++mediaListPos.index < currentFiles.count()) {
                libvlc_media_list_player_next(mediaListPlayer);
                emit currentFileName(currentFiles[mediaListPos.index]);
                slFilesIndex->setValue(mediaListPos.index);
                if ( sizes.count() == 0) {
                    mediaLength = 0;
                }
            } else {
                stopIntern();
            }
        }

    }
}
void Player::setFilesIndex(int index){
    if ( playerRuns ) {
        mediaListPos.index = index;
        libvlc_media_list_player_play_item_at_index(mediaListPlayer,index);
        emit currentFileName(currentFiles[mediaListPos.index]);
        slFilesIndex->setValue(mediaListPos.index);
        if ( sizes.count() == 0) {
            mediaLength = 0;
        }
    }
}

void Player::setPositionFloat(float pos){
    if (playerRuns) {
        libvlc_media_player_set_position (mediaPlayer,pos);
    }
}
void Player::setPositionInt(int pos){
    setPositionFloat((float)(pos)/maxValue);
}

void Player::skipFileOfVideo(int value){ // -1 0 +1
    if (( value < 0 ) & (mediaListPos.index > 0)) {
        setFilesIndex(mediaListPos.index -1);
    } else if (( value > 0 ) & (mediaListPos.index < currentFiles.count()-1)) {
        setFilesIndex(mediaListPos.index + 1);
    }
}
void Player::skipToPos(int value){
    if ( !isRunningVideo() ) return;
    qint64 pos = value*1000;
    qint64 mark = 0;
    qint64 length = 0;
    int index = 0;

    for (int i = 0; i < sizes.count(); i++) {
        if ( mark+sizes[i] <= pos) {
            mark += sizes[i];
            index = i+1;
        } else {
            break;
        }
    }
    length = sizes[index];
    pos -= mark;
    if ( index != mediaListPos.index ) {
        setFilesIndex(index);
    }
    setPositionFloat((float)(pos) / length);
}
void Player::skipPosOfVideo(int value){ // -1 0 +1
    int64_t oneMinute = maxValue / ( mediaLength / 60000 );
    int newTimeValue = slPosition->value() + (oneMinute * value);
    if ( newTimeValue < 0 ) newTimeValue = 0;
    if ( newTimeValue > maxValue ) newTimeValue = maxValue -50;
    setPositionInt(newTimeValue);
}

/*
    SET Control Panel Widgets +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

void Player::setControls(QSlider *vol, ClickSlider *pos, QSlider *index, QLabel *t, QLabel *tPos) {
    slVolume = vol;
    slPosition = pos;
    slFilesIndex = index;
    lTime = t;
    lTimeLeft = tPos;
    connect(slPosition,SIGNAL(sliderMoved(int)),this,SLOT(setPositionInt(int)));
    connect(slFilesIndex,SIGNAL(valueChanged(int)),this,SLOT(setFilesIndex(int)));
    connect(slVolume,SIGNAL(valueChanged(int)),this,SLOT(setVolume(int)));
}

void Player::setMediaSliderPosition(float newValue){
    int newPos = (int) (newValue * maxValue);
    slPosition->setValue(newPos);
}

int Player::videoLength(void){
    qint64 length = 0;
    foreach(int size,sizes) {
        length += size;
    }
    if ( length ) {
        return length/1000;
    }
    return MAXVALUEDEFAULT;
}

int Player::videoPos(void){
    qint64 length = 0;
    if ( isRunningVideo() ) {
        length += libvlc_media_player_get_time (mediaPlayer);
        if ( sizes.count() ) {
            for ( int i = 0; i < mediaListPos.index; i++ ) {
                length += sizes[i];
            }
            lTimeLeft->setText( timeToString(mediaLength - length));
            return length /1000;
        } else {
            lTimeLeft->setText( timeToString(mediaLength - length));
        }
    }
    return slPosition->value();
}
