#include <QFont>
#include <QVariant>
#include <QTextCursor>
#include <QTextCodec>
#include "recmodel.h"
#include "func.h"


static int columnToSort;

RecModel::RecModel(Optionen *mConf,QObject *parent) : QAbstractTableModel(parent) {
    conf = mConf;
    fontSize = 11;
#ifdef RECLOAD
    load();
#endif
}
RecModel::~RecModel() {
#ifdef RECLOAD
    save();
#endif
}
void RecModel::save(void){
    QString text = "";
    for ( int i = 0; i < dataTable.count(); i++) {
        QString line = dataTable[i]->join("++");
        line.replace("\n","<br />");
        text += line+"\n";
    }
    saveToFile(getIniPath()+"/rec.txt",text);
}
void RecModel::load(void){
    QStringList list = loadFromFile(getIniPath()+"/rec.txt").split("\n");
    if ( list.count() > 1)  {
        for ( int i = 0; i < list.count(); i++) {
            // if ( list[i].size()  > 4 ) {
            list[i].replace("<br />","\n");
            QStringList *record = new QStringList(list[i].split("++"));
            while ( record->count() <= REC_END ) {
                // expand to new columns
                record->append("");
            }
            dataTableNew << record;
            //logos->findLogo(record->at(REC_CHNAME),record->at(REC_CHNAME));
            // }
        }
    }

}

void RecModel::setLogos(TvLogo *logoClass){
    logos = logoClass;
}

void RecModel::dataReady(void){
#ifdef RECLOAD
    clearTable();
    foreach ( QStringList *list,dataTableNew ) {
        dataTable << list;
    }
    dataTableNew.clear();
#endif
    emit layoutChanged();
}

int RecModel::createVideoList(QString path,QStringList *list,QStringList *marks){
    list->clear();
    marks->clear();
    QDir dir(path);
    QStringList files = dir.entryList();
    for ( int i = 0; i < files.count(); i++) {
        if (files[i].left(1) == "0") {
            QString fileName = path+"/"+files[i];
#ifdef Q_OS_MAC
            fileName = QDir::toNativeSeparators ( fileName );
#endif

           list->append(fileName);
            conf->logger("recmodel.cpp->"+fileName);
        }
        if (( files[i].toUpper() == "MARKS.VDR") | ( files[i].toUpper() == "MARKS"))        {
            QString fileName = path+"/"+files[i];
            

#ifdef Q_OS_WIN
            fileName = QDir::toNativeSeparators ( fileName );
#endif
            marks->append( loadFromFile(fileName).split("\n") );
        }
    }
    return list->count();
}

void RecModel::clearTable(void){
    conf->logger("recmodel.cpp->clearTable()");
    while (dataTable.count() ) {
        delete dataTable.takeLast();
    }
}

void RecModel::setHeader(QStringList newHeader){
    emit layoutAboutToBeChanged();
    foreach (QString h,newHeader) {
        headers << h;
    }
    emit layoutChanged();
}
QVariant RecModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if(orientation == Qt::Vertical ){
        if (role == Qt::DisplayRole) {
            return section;
        }
    } else {
        if (role == Qt::DisplayRole) {
            if ( section < 0) return section;
            if ( section < headers.count() ) return headers[section];
            return section;
        } else if ( role == Qt::FontRole ) {
            return QFont(conf->eFont->text(),fontSize,QFont::Bold);
        }
    }
    return QVariant();
}
QVariant RecModel::data( const QModelIndex & index, int role ) const {
    //fontSize*5
    if ( index.isValid() ) {
        if ( role == getRowRole ) {
            return index.row();
        }
        if ( index.column() == REC_NAME ) {
            QString text = dataTable[index.row()]->at(index.column());
            int pos = text.indexOf("\n");
            if ((role == headerTextRole) | (role == sortingRole)) {
                return text.left(pos);
            } else if (role == subHeaderTextRole) {
                return text.mid(pos+1);
            //} else if (role == iconRole) {
            //    return QIcon(":img/images/easyStream100.png");
            }
         } else {
            if ( (role == Qt::DisplayRole) | (role == sortingRole)) {
                return dataTable[index.row()]->at(index.column());
            } else if ( role == Qt::DecorationRole) {
                if ( index.column()  == REC_CHNAME ) {
                    return logos->getLogo(dataTable[index.row()]->at(index.column()),"");
                }
            } else if ( role == Qt::FontRole ) {
                if ( index.column()  < 3 ) {
                    return QFont(conf->eFont->text(),fontSize * 1.3);
                } else {
                    return QFont(conf->eFont->text(),fontSize);
                }
            } else if ( role == Qt::TextAlignmentRole ) {
                switch ( index.column() ) {
                case 0: return Qt::AlignHCenter + Qt::AlignVCenter;
                case 1: return Qt::AlignHCenter + Qt::AlignVCenter;
                default: return Qt::AlignVCenter;
                }
            }
        }
    }
    return QVariant();
}
int RecModel::columnCount(const QModelIndex & /*parent*/ ) const {
    return headers.count();
}
int RecModel::rowCount(const QModelIndex & /*parent*/ ) const {
    return dataTable.count();
}
QString RecModel::text(int row, int column){
    return dataTable[row]->at(column);
}
QString RecModel::text(QModelIndexList list, int column){
//    QModelIndexList list = selectionModel()->selectedIndexes();
    return list[column].data(sortingRole).toString();
}
QString RecModel::fullPath(int row) {
//    QString path =  rootPath+"/"+dataTable[row]->at(REC_PATH);
//    return path.replace("//","/");
    return dataTable[row]->at(REC_FULLPATH);
}
void RecModel::setRootPath(QString path){
    if ( path.right(1) != QDir::separator()) {
        path += QDir::separator();
    }
    rootPath = path;
}

void RecModel::addRec(QString path,QString parentPath,int index){
    QString space = "-----------------------------------------------------";
    conf->logger("recmodel.cpp->addRec() "+space.left(index)+path,2);
//    if ( index > 1) {
//        QString folder = path.mid(rootPath.size());
//        int pos = folder.indexOf("/");
//        folder = folder.left(pos);
//        emit groupFound(folder);
//    }
    if ( conf ) {
       fontSize = conf->tBFontSize->value();
    }
    QString infoName;
    int sizeCalc;
    int sizeInfo;
    QDir dir(path);
    foreach (QFileInfo fileInfo ,dir.entryInfoList(QDir::Files) ) {
        if (fileInfo.baseName() == "info") infoName = fileInfo.absoluteFilePath();
        if (fileInfo.baseName() == "index") sizeCalc = fileInfo.size() / (200); // 8 Bytes / Frame -- 25 Frames / sec
    }
    QString infoStr = "";
    if ( infoName.size() ) {
        if ( infoName.right(4) == ".vdr") {
            infoStr = loadFromFile(infoName,"ISO 8859-15");
        } else {
            infoStr = loadFromFile(infoName);
        }
    }
    QStringList infoList = infoStr.split("\n");

    //int row = dataTable.count();
    QStringList *list = new QStringList();
#ifdef RECLOAD
    dataTableNew << list;
#else
    dataTable << list;
    emit layoutChanged();
#endif

    QStringList zeit;
    QString title;
    QString dateStr;
    QString content = "";
    QString channelName = "";

    if ( infoList.count() == 1) {
        // keine Infodatei gefunden
        int pos = parentPath.lastIndexOf("/");
        title = parentPath.mid(pos+1);
        pos = path.lastIndexOf("/");
        // 2012-09-19.20.13.8-0.rec
        dateStr = path.mid(pos+1).replace(".rec","",Qt::CaseInsensitive);
        pos = dateStr.lastIndexOf(".");
        if ( pos > -1 ) dateStr = dateStr.left(pos);
        pos = dateStr.indexOf(".");
        if ( pos > -1 ) dateStr[pos] = '\n';
        dateStr = dateStr.left(pos+6);
        dateStr.replace('.',':');
    } else {
        channelName = readFromList(&infoList,"C");
        int pos = channelName.indexOf(" ");
        channelName = channelName.mid(pos+1);
        title = readFromList(&infoList,"T");
        QString subTitle = readFromList(&infoList,"S");
        if ( subTitle.size() ) title += "\n"+subTitle;
        content = readFromList(&infoList,"D");

        zeit = readFromList(&infoList,"E").split(" ");
        
        if ( zeit.size() > 1) {
            sizeInfo = zeit.at(2).toInt();
            QDateTime dateTime = QDateTime::fromTime_t(zeit.at(1).toInt());
            dateStr = dateTime.toString(tr("yyyy-MM-dd\nhh:mm"));
        } else {
            sizeInfo = 0 ;
            QDateTime dateTime = QDateTime::fromTime_t(0);
            dateStr = dateTime.toString(tr("yyyy-MM-dd\nhh:mm"));
        }
    }
    // Bei HD sind es 50 Frames/Sec -> größe halbieren
    if ( channelName.indexOf("HD") > -1) {
        sizeCalc /=2;
    }
    if ( sizeCalc > sizeInfo*2) {
        sizeCalc /=2;
    }
    QString dauer = QString::number(sizeCalc / 60)+" / "+QString::number(sizeInfo/60);;
//    REC_DATE,
//    REC_TIME,
//    REC_NAME,
//    REC_CONTENT,
//    REC_PATH,
//    REC_FILES,
//    REC_RECID

    list->append(dateStr); // << dateStr;  //1.3 Qt::AlignCenter | Qt::AlignVCenter
    list->append(dauer); // << dauer;    //1.3 Qt::AlignRight | Qt::AlignVCenter
    list->append(title); // << title;    //1.3
    list->append(content); // << content;
    list->append(channelName); // << infoName;
    list->append(path); // << path;
    list->append(infoName); // << infoName;
    list->append(parentPath); // << infoName;

//    ui->tWChannels->setRowHeight(row,fontSize*5);

}
void RecModel::addFolder(QString path,QString parentPath,int count){

    conf->logger("recmodel.cpp->addFolder() "+path,2);
    if ( conf ) {
       fontSize = conf->tBFontSize->value();
    }
//    REC_DATE,
//    REC_TIME,
//    REC_NAME,
//    REC_CONTENT,
//    REC_PATH,
//    REC_FILES,
//    REC_RECID
    //int row = dataTable.count();
    QStringList *list = new QStringList();
#ifdef RECLOAD
    dataTableNew << list;
#else
    dataTable << list;
    emit layoutChanged();
#endif

    list->append("("+QString::number(count)+")"); // << dateStr;  //1.3 Qt::AlignCenter | Qt::AlignVCenter
    list->append(""); // << dauer;    //1.3 Qt::AlignRight | Qt::AlignVCenter
    list->append(" "+path); // << title;    //1.3
    list->append(""); // << content;
    list->append(""); // << infoName;
    list->append(path); // << path;
    list->append(""); // << infoName;
    list->append(parentPath); // << fullPath;

//    ui->tWChannels->setRowHeight(row,fontSize*5);
}
bool lessThan(const QStringList *s1,const QStringList *s2){
    return s1->at(columnToSort).toLower() < s2->at(columnToSort).toLower();
}
bool moreThan(const QStringList *s1,const QStringList *s2){
    return s1->at(columnToSort).toLower() > s2->at(columnToSort).toLower();
}

void RecModel::sort ( int column, Qt::SortOrder order){
    //QList<QStringList *> dataTable;
    columnToSort = column;
    if ( order == Qt::AscendingOrder) {
        qStableSort(dataTable.begin(),dataTable.end(),lessThan);
    } else {
        qStableSort(dataTable.begin(),dataTable.end(),moreThan);
    }
    emit layoutChanged();
}


